% funkce na hledani christoffelovych cisel h, viz vztah (6.39) na str. 60
%author: Patrik Drda
function [h,p_r]=christ_num(r,m,n,lambda_0,lambda_r,mu_r,roots_search,L)

% vypocet L_r, M_r podle vztahu (6.6), str. 52
L_r= @(n) prod([lambda_0 lambda_r(1:n)]); 
M_r=@(n) prod([1 mu_r(1:n)]);

if r <= m
     K=m+n;
else
    K=r+n;
end

% vyber metody na hledani korenu 
switch roots_search
    case 1
[roots,R]=roots_method1(K,lambda_0,lambda_r,mu_r); % zavolani metody na vypocet korenu pomoci matice a jeji vl. cisel
    case 2
[roots,R]=roots_method2(L,K,lambda_0,lambda_r,mu_r); % QD-algpritmus
end
   %% vypocet B'_r,n (derivace), vztah 6.39 str. 61
   
   % variace indexu
    a=fliplr(fullfact([R R]));
    a(~diff(a')',:)=[];
   
    for k=1:length(a)
       ck(k)=(roots(a(k,1))-roots(a(k,2))); % rozdil korenu (**)
    end
    
 % algoritmus na hledani soucinu rozdilu (**)   
 Q=zeros(R,R); 
for l=1:length(ck)/(K-1)
    for ll=1:length(ck)/(K-1)
        if l==1 && ll==1
            Q(1,1)=prod(ck(1:(K-1)));
        else
        Q(l,ll)=prod(ck(1+((l-1)*(K-1)):(K-1)*ll)); % prvky na diagonale jsou vysledky daneho soucinu rozdilu (**)
        end
    end
end
    
%% vypocet A(s) a B(s) pomoci rekurence
% A_1(s)=1, A_2(s)=(lambda1+mu1+s),
% A_j(s)=(lambda_j-1+mu_j-1+s)*A_j-1(s)-lambda_j-2*mu_j-2*A_j-2(s)
% stejny postup pro B
% rekurentni vztahy nalezneme na strane 61, vztahy (6.40) a (6.41)
A=zeros(R+1,R);
B=zeros(R+1,R);
   for jj=1:R+1
     for dd=1:R
       if jj==1 
           A(jj,dd)=0;
           B(jj,dd)=1;
       elseif jj==2
           A(jj,dd)=1;
           B(jj,dd)=lambda_0+roots(dd);
       else
            A(3,dd)=(lambda_r(3-2)+mu_r(3-2)+roots(dd))*A(3-1,dd)-lambda_0*mu_r(3-2)*A(3-2,dd);
            A(jj+1,dd)=(lambda_r(jj+1-2)+mu_r(jj+1-2)+roots(dd))*A(jj+1-1,dd)-lambda_r(jj+1-3)*mu_r(jj+1-2)*A(jj+1-2,dd);
            
            B(3,dd)=(lambda_r(1)+mu_r(1)+roots(dd))*B(2,dd)-lambda_0*mu_r(1);
            B(jj+1,dd)=(lambda_r(jj+1-2)+mu_r(jj+1-2)+roots(dd))*B(jj+1-1,dd)-lambda_r(jj+1-3)*mu_r(jj+1-2)*B(jj+1-2,dd);
       end
     end 
   end
   
   %% vypocet h
   h=zeros(1,n);
   for ff=1:R
       if m==0 
        h(ff)= (B(r+1,ff)*A(R+1,ff))/(Q(ff,ff)*M_r(r));  % vzorec pro sla h (str. 60, vztah (6.39)
       else
            h(ff)= (B(r+1,ff)*B(m+1,ff)*A(R+1,ff))/(Q(ff,ff)*M_r(r)*L_r(m-1)); 
       end
   end
   
%% vypocet pravdebodnosti
p_r=@(t) h(1:K)*exp(roots(1:K)*t); % inverzni Laplaceova transformace , vztah (6.37) , str. 59
